<?php
//
// Linux Server Config Wizard
// Copyright (c) 2010-2018 Nagios Enterprises, LLC. All rights reserved.
//

include_once(dirname(__FILE__) . '/../configwizardhelper.inc.php');

ec2_configwizard_init();

function ec2_configwizard_init()
{
    $name = "ec2";
    $args = array(
        CONFIGWIZARD_NAME => $name,
        CONFIGWIZARD_VERSION => "1.0.0",
        CONFIGWIZARD_TYPE => CONFIGWIZARD_TYPE_MONITORING,
        CONFIGWIZARD_DESCRIPTION => _("Monitor an Amazon EC2 Instance."),
        CONFIGWIZARD_DISPLAYTITLE => "Amazon EC2",
        CONFIGWIZARD_FUNCTION => "ec2_configwizard_func",
        CONFIGWIZARD_PREVIEWIMAGE => "ec2.png",
        CONFIGWIZARD_FILTER_GROUPS => array('amazon'),
        CONFIGWIZARD_REQUIRES_VERSION => 500
    );
    register_configwizard($name, $args);
}

/**
 * @return int
 *          0 = good to go
 *          1 = prereqs non-existent
 *          2 = may need to upgrade boto3
 */

function ec2_configwizard_check_prereqs()
{
    // Plugin doesn't exist
    if (!file_exists("/usr/local/nagios/libexec/check_ec2.py")) {
        return 4; // plugin non-existent
    }

    $preferred_version = '1.4.7';

    $found_pip = false;
    $pip_output = array();

    // See if boto3 is installed via pip and get the version number
    // $pip_command = "pip show boto3"; <-- THIS HAS LOW PERFORMANCE SOMETIMES, DO NOT USE
    $pip_command = 'python -c "import boto3"';
    exec($pip_command, $pip_output, $pip_return);

    // If neither yum nor pip returned anything, there is no need to continue
    if ($pip_return !== 0) {
        return 1; // prereqs non-existent
    }
}

/**
 * @param string $mode
 * @param null   $inargs
 * @param        $outargs
 * @param        $result
 *
 * @return string
 */

function ec2_configwizard_func($mode = "", $inargs = null, &$outargs, &$result)
{

    $wizard_name = "ec2";
    $local_url = get_base_url();

    // Initialize return code and output
    $result = 0;
    $output = "";

    // Initialize output args - pass back the same data we got
    $outargs[CONFIGWIZARD_PASSBACK_DATA] = $inargs;

    switch ($mode) {

        case CONFIGWIZARD_MODE_GETSTAGE1HTML:
            
            $check_prereqs = ec2_configwizard_check_prereqs();

            if ($check_prereqs == 1) {
                $output .= '<div class="message errorMessage" style="padding: 10px; margin-bottom: 20px;"><p><b>' . _('Error') . ':</b> ' . _('It looks like you are missing boto3 on your Nagios XI server.') . '</p><p>' . _('To use this wizard you must install boto3 on your server. If you are using CentOS or RHEL you can run:') . '</p><pre>pip install boto3</pre></div>';

            } elseif ($check_prereqs == 4) {
                $output .= '<div class="message errorMessage" style="padding: 10px; margin-bottom: 20px;"><p><b>' . _('Error') . ':</b> ' . _('It looks like you are missing the check_ec2.py plugin on your Nagios XI server.</p></div>');
            } else {

            $address = grab_array_var($inargs, "address", "");
            $address = nagiosccm_replace_user_macros($address);

            $instanceid = grab_array_var($inargs, "instanceid", "");
            $accesskeyid = grab_array_var($inargs, "accesskeyid", "");
            $accesskey = grab_array_var($inargs, "accesskey", "");
            $staticcreds = grab_array_var($inargs, "staticcreds", "");
            $staticconfig = grab_array_var($inargs, "staticconfig", "");
            $credsfilepath = grab_array_var($inargs, "credsfilepath", "");
            $configfilepath = grab_array_var($inargs, "configfilepath", "");
            $region = grab_array_var($inargs, "region", "");

            if ($credsfilepath == "") {
                $credsfilepath = "/home/nagios/.aws/credentials";
            }

            if ($configfilepath == "") {
                $configfilepath = "/home/nagios/.aws/config";
            }

            $linuxdistro = grab_array_var($inargs, "linuxdistro", "");

            $output = '
            <h5 class="ul">'._('EC2 Instance Information').'</h5>
            <table class="table table-condensed table-no-border table-auto-width table-padded">
                <tr>
                    <td class="vt" style="width: 135px;">
                        <label for="address">'._('IP Address').':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="address" id="address" value="'.encode_form_val($address).'" class="textfield form-control">
                        <div class="subtext">'._("The IP address or FQDNS name of the EC2 instance you'd like to monitor").'.</div>
                    </td>
                </tr>
                <tr>
                    <td class="vt" style="width: 135px;">
                        <label>' . _('Instance ID') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="instanceid" id="instanceid" value="' . encode_form_val($instanceid) . '" class="textfield form-control">
                        <div class="subtext">' . _('The unique ID of the EC2 instance to be monitored') . '.</div>
                    </td>
                </tr>
            </table>
            <h5 class="ul">'._('EC2 Credentials').'</h5>
            <p>' . _('Specify the credentials for the EC2 instance you would like to monitor.') . '</p>
            <table class="table table-condensed table-no-border table-auto-width">
                <tr>
                    <td class="vt" style="width: 135px;">
                        <label>' . _('Access Key ID') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="accesskeyid" id="accesskeyid" value="' . encode_form_val($accesskeyid) . '" class="textfield form-control credinput" ' . (checkbox_binary($staticcreds) ? "disabled" : "") . '>
                        <div class="subtext">' . _('The Access Key ID of the instance to be monitored') . '.</div>
                    </td>
                </tr>
                <tr>
                    <td class="vt" style="width: 135px;">
                        <label>' . _('Secret Access Key') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="accesskey" id="accesskey" value="' . encode_form_val($accesskey) . '" class="textfield form-control credinput" ' . (checkbox_binary($staticcreds) ? "disabled" : "") . '>
                        <div class="subtext">' . _('The Secret Access Key of the instance to be monitored') . '.</div>
                    </td>
                </tr>
            </table>

            <table class="table table-condensed table-no-border table-auto-width">
                <tr>
                    <td style="width: 135px;"></td>
                    <td>
                        <input type="checkbox" class="checkbox" id="staticcreds" name="staticcreds" ' . is_checked(checkbox_binary($staticcreds), "1") . '>
                    </td>
                    <td>
                        <label class="normal" for="staticcreds">
                            <b>' . _('Static credentials file') . '</b><br>
                            ' . _('Use a statically defined AWS credentials file') . '.
                        </label>

                        <i class="credtooltip fa fa-question-circle fa-14" data-placement="right" data-content="If this box is checked, Nagios will read a local file containing your AWS credentials that follows Amazon\'s syntax.  Read more about credentials files <a target=\'' . '_blank' . '\' href=\'' . 'http://docs.aws.amazon.com/cli/latest/userguide/cli-config-files.html' . '\'>here</a>.">
                    </td>
                </tr>
            </table>


            <script type="text/javascript">

            $(function(){
                $(".credtooltip").popover({ html: true });
            });

            </script>


            <table class="table table-condensed table-no-border table-auto-width ' . (checkbox_binary($staticcreds) ? "" : "hidden") . '" id="credsfiletable">
                <tr>
                    <td class="vt" id="filepath" style="width: 135px;">
                        <label>' . _('File path') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="credsfilepath" id="credsfilepath" value="' . encode_form_val($credsfilepath) . '" class="textfield form-control">
                        <div class="subtext">' . _('The absolute file path to the Amazon Credentials file (<b>Default:</b> /home/nagios/.aws/credentials)') . '.</div>
                    </td>
                </tr>
            </table>

            <script>

                document.addEventListener("DOMContentLoaded", function() {
                    var credtable = document.getElementById("credsfiletable");
                    var credinputs = document.getElementsByClassName("credinput");

                    document.querySelector("#staticcreds").addEventListener("change", function() {
                        if (credtable.classList.contains("hidden")) {
                            credtable.classList.remove("hidden");
                        } else {
                            credtable.classList.add("hidden");
                        }

                        for (var i = 0; i < credinputs.length; i++) {
                            if (credinputs[i].disabled === true) {
                                credinputs[i].disabled = false;
                            } else {
                                credinputs[i].disabled = true;
                            }
                        }
                    });
                });
            </script>

            <h5 class="ul">'._('EC2 Configuration').'</h5>
            <p>' . _('Specify the credentials for the EC2 instance you would like to monitor.') . '</p>
            <table class="table table-condensed table-no-border table-auto-width">
                <tr>
                    <td class="vt" style="width: 135px;">
                        <label>' . _('Region') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" placeholder="us-east-1" name="region" id="region" value="' . encode_form_val($region) . '" class="textfield form-control configinputs" ' . (checkbox_binary($staticconfig) ? "disabled" : "") . '>
                        <div class="subtext">' . _('The region name of the EC2 instance to be monitored') . '.</div>
                    </td>
                </tr>
            </table>

            <table class="table table-condensed table-no-border table-auto-width">
                <tr>
                    <td style="width: 135px;"></td>
                    <td>
                        <input type="checkbox" class="checkbox" id="staticconfig" name="staticconfig" ' . is_checked(checkbox_binary($staticconfig), "1") . '>
                    </td>
                    <td>
                        <label class="normal" for="staticconfig">
                            <b>' . _('Static configuration file') . '</b><br>
                            ' . _('Use a statically defined AWS configurations file') . '.
                        </label>
                        <i class="credtooltip fa fa-question-circle fa-14" data-placement="right" data-content="If this box is checked, Nagios will read a local file containing your AWS configuration that follows Amazon\'s syntax.  Read more about configuration files <a target=\'' . '_blank' . '\' href=\'' . 'http://docs.aws.amazon.com/cli/latest/userguide/cli-config-files.html' . '\'>here</a>.">
                    </td>
                    
                </tr>
            </table>

            <script type="text/javascript">

            $(function(){
                $(".configtooltip").popover({ html: true });
            });

            </script>

            <table class="table table-condensed table-no-border table-auto-width ' . (checkbox_binary($staticconfig) ? "" : "hidden") . '" id="configfiletable">
                <tr>
                    <td class="vt" style="width: 135px;">
                        <label>' . _('File path') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="configfilepath" id="configfilepath" value="' . encode_form_val($configfilepath) . '" class="textfield form-control">
                        <div class="subtext">' . _('The absolute file path to the Amazon Configuration file (<b>Default:</b> /home/nagios/.aws/config)') . '.</div>
                    </td>
                </tr>
            </table>

            <script>

                document.addEventListener("DOMContentLoaded", function() {
                    var configtable = document.getElementById("configfiletable");
                    var configinputs = document.getElementsByClassName("configinputs");

                    document.querySelector("#staticconfig").addEventListener("change", function() {
                        if (configtable.classList.contains("hidden")) {
                            configtable.classList.remove("hidden");
                        } else {
                            configtable.classList.add("hidden");
                        }

                        for (var i = 0; i < configinputs.length; i++) {
                            if (configinputs[i].disabled === true) {
                                configinputs[i].disabled = false;
                            } else {
                                configinputs[i].disabled = true;
                            }
                        }
                    });
                });

            </script>';
            }
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE1DATA:

            // Get variables that were passed to us
            $instanceid = grab_array_var($inargs, "instanceid", "");
            $accesskeyid = grab_array_var($inargs, "accesskeyid", "");
            $accesskey = grab_array_var($inargs, "accesskey", "");
            $credsfilepath = grab_array_var($inargs, "credsfilepath", "");
            $configfilepath = grab_array_var($inargs, "configfilepath", "");
            $region = grab_array_var($inargs, "region", "");
            $address = grab_array_var($inargs, "address", "");

            $staticcreds = grab_array_var($inargs, "staticcreds", "off");
            $staticconfig = grab_array_var($inargs, "staticconfig", "off");

            // Check for errors
            $errors = 0;
            $errmsg = array();
            if (empty($address)) {
                $errmsg[$errors++] = "No address specified.";
            }

            if (empty($accesskeyid) and $staticcreds == "off") {
                $errmsg[$errors++] = "Specify either an Access Key ID or a static credentials file.";
            }

            if (empty($accesskey) and $staticcreds == "off") {
                $errmsg[$errors++] = "Specify either a Secret Access Key or a static credentials file.";
            }

            if (empty($region) and $staticconfig == "off") {
                $errmsg[$errors++] = "Specify either a region or a static configuration file.";
            }

            if (empty($instanceid)) {
                $errmsg[$errors++] = "No instance ID specified.";
            }

            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }
            break;

        case CONFIGWIZARD_MODE_GETSTAGE2HTML:

            // Get variables that were passed to us
            $instanceid = grab_array_var($inargs, "instanceid", "");
            $accesskeyid = grab_array_var($inargs, "accesskeyid", "");
            $accesskey = grab_array_var($inargs, "accesskey", "");
            $staticcreds = grab_array_var($inargs, "staticcreds", "");
            $staticconfig = grab_array_var($inargs, "staticconfig", "");
            $credsfilepath = grab_array_var($inargs, "credsfilepath", "");
            $configfilepath = grab_array_var($inargs, "configfilepath", "");
            $region = grab_array_var($inargs, "region", "");
            $address = grab_array_var($inargs, "address", "");

            $ha = @gethostbyaddr($address);
            if (empty($ha)) {
                $ha = $address;
            }
            
            $hostname = grab_array_var($inargs, "hostname", $ha);
            $hostname = nagiosccm_replace_user_macros($hostname);

            $password = "";

            $services = "";
            $services_serial = grab_array_var($inargs, "services_serial", "");
            if ($services_serial != "") {
                $services = unserialize(base64_decode($services_serial));
            }
            if (!is_array($services)) {
                $services_default = array(
                    "ping" => 1,
                    "CPUCreditUsage" => 1,
                    "CPUCreditBalance" => 1,
                    "CPUUtilization" => 1,
                    "DiskReadOps" => 1,
                    "DiskWriteOps" => 1,
                    "DiskReadBytes" => 1,
                    "DiskWriteBytes" => 1,
                    "NetworkIn" => 1,
                    "NetworkOut" => 1,
                    "NetworkPacketsIn" => 1,
                    "NetworkPacketsOut" => 1,
                    "StatusCheckFailed" => 1,
                    "StatusCheckFailed_Instance" => 1,
                    "StatusCheckFailed_System" => 1

                );
                $services_default["servicestate"][0] = "on";
                $services_default["servicestate"][1] = "on";
                $services = grab_array_var($inargs, "services", $services_default);
            }

            $serviceargs = "";
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", "");
            if ($serviceargs_serial != "") {
                $serviceargs = unserialize(base64_decode($serviceargs_serial));
            }
            if (!is_array($serviceargs)) {
                $serviceargs_default = array(

                    "CPUCreditUsage_warning" => 60,
                    "CPUCreditUsage_critical" => 120,

                    "CPUCreditBalance_warning" => 100,
                    "CPUCreditBalance_critical" => 25,

                    "CPUUtilization_warning" => 50,
                    "CPUUtilization_critical" => 75,
                    
                    "DiskReadOps_warning" => 15000,
                    "DiskReadOps_critical" => 30000,
                    
                    "DiskWriteOps_warning" => 15000,
                    "DiskWriteOps_critical" => 30000,
                    
                    "DiskReadBytes_warning" => 1000000000,
                    "DiskReadBytes_critical" => 2000000000,
                    
                    "DiskWriteBytes_warning" => 1000000000,
                    "DiskWriteBytes_critical" => 2000000000,
                    
                    "NetworkIn_warning" => 1000000000,
                    "NetworkIn_critical" => 2000000000,
                    
                    "NetworkOut_warning" => 1000000000,
                    "NetworkOut_critical" => 2000000000,
                    
                    "NetworkPacketsIn_warning" => 1750000,
                    "NetworkPacketsIn_critical" => 3500000,
                    
                    "NetworkPacketsOut_warning" => 1750000,
                    "NetworkPacketsOut_critical" => 3500000,
                    
                    "StatusCheckFailed_warning" => 1,
                    "StatusCheckFailed_critical" => 1,
                    
                    "StatusCheckFailed_Instance_warning" => 1,
                    "StatusCheckFailed_Instance_critical" => 1,
                    
                    "StatusCheckFailed_System_warning" => 1,
                    "StatusCheckFailed_System_critical" => 1,
                );

                $serviceargs = grab_array_var($inargs, "serviceargs", $serviceargs_default);
            }

            $output = '
            <input type="hidden" name="address" value="' . encode_form_val($address) . '">
            <input type="hidden" name="hostname" value="' . encode_form_val($hostname) . '">
            <input type="hidden" name="instanceid" value="' . encode_form_val($instanceid) . '">
            <input type="hidden" name="accesskeyid" value="' . encode_form_val($accesskeyid) . '">
            <input type="hidden" name="accesskey" value="' . encode_form_val($accesskey) . '">
            <input type="hidden" name="region" value="' . encode_form_val($region) . '">
            <input type="hidden" name="staticcreds" value="' . encode_form_val($staticcreds) . '">
            <input type="hidden" name="staticconfig" value="' . encode_form_val($staticconfig) . '">
            <input type="hidden" name="credsfilepath" value="' . encode_form_val($credsfilepath) . '">
            <input type="hidden" name="configfilepath" value="' . encode_form_val($configfilepath) . '">

            <h5 class="ul">' . _('EC2 Instance Details') . '</h5>
            <table class="table table-condensed table-no-border table-auto-width table-padded">
                <tr>
                    <td class="vt" style="width: 170px;">
                        <label for="address">' . _('IP Address') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="address" id="address" value="' . encode_form_val($address) . '" class="textfield form-control" readonly>
                    </td>
                </tr>
                <tr>
                    <td class="vt" style="width: 170px;">
                        <label for="hostname">' . _('Host Name') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="hostname" id="hostname" value="' . encode_form_val($hostname) . '" class="textfield form-control">
                        <div class="subtext">' . _("The name you'd like to have associated with this Linux server") . '.</div>
                    </td>
                </tr>
            </table>

            <h5 class="ul">' . _('EC2 Instance Credentials') . '</h5>
            <table class="table table-condensed table-no-border table-auto-width">';

            if ($staticcreds == "on") {
                $output .= '
                <tr class="credentialsfilepath">
                    <td class="vt" style="width: 170px;">
                        <label>' . _('Credentials File Path') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="credsfilepath" id="credsfilepath" value="' . encode_form_val($credsfilepath) . '" class="textfield form-control credsinput" disabled>
                    </td>
                </tr>';
            } else {

                $output .=
                '<tr class="specifiedcredentials">
                    <td class="vt" style="width: 170px;">
                        <label>' . _('Instance ID') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="instanceid" id="instanceid" value="' . encode_form_val($instanceid) . '" class="textfield form-control credsinput" disabled>
                    </td>
                </tr>
                <tr class="specifiedcredentials">
                    <td class="vt" style="width: 170px;">
                        <label>' . _('Access Key ID') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="accesskeyid" id="accesskeyid" value="' . encode_form_val($accesskeyid) . '" class="textfield form-control credsinput" disabled>
                    </td>
                </tr>
                <tr class="specifiedcredentials">
                    <td class="vt" style="width: 170px;">
                        <label>' . _('Secret Access Key') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="accesskey" id="accesskey" value="' . encode_form_val($accesskey) . '" class="textfield form-control credsinput" disabled>
                    </td>
                </tr>';
            }

            $output .= '    
            </table>

            <h5 class="ul">' . _('EC2 Instance Configuration') . '</h5>
            <table class="table table-condensed table-no-border table-auto-width">';

            if ($staticconfig == "on") {
                $output .= '
                <tr class="configurationfilepath">
                    <td class="vt" style="width: 170px;">
                        <label>' . _('Configuration File Path') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="configfilepath" id="configfilepath" value="' . encode_form_val($configfilepath) . '" class="textfield form-control credsinput" disabled>
                    </td>
                </tr>';
            } else {
                $output .= '
                <tr class="specifiedconfiguration">
                    <td class="vt" style="width: 170px;">
                        <label>' . _('Region') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="region" id="region" value="' . encode_form_val($region) . '" class="textfield form-control" disabled>
                    </td>
                </tr>';
            }

            $output .= '
            </table>

            <h5 class="ul">' . _('Instance Metrics') . '</h5>
            <p>' . _("Specify which services you'd like to monitor for the EC2 Instance") . '.</p>
            <table class="table table-no-border table-auto-width table-padded">
                <tr>
                    <td>
                        <input type="checkbox" class="checkbox" id="ping" name="services[ping]"  ' . is_checked(checkbox_binary($services["ping"]), "1") . '>
                    </td>
                    <td>
                        <label for="ping" class="select-cf-option">' . _('Ping') . '</label><br>
                        ' . _('Monitors the server with an ICMP ping.  Useful for watching network latency and general uptime') . '.
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" class="checkbox" id="CPUCreditUsage" name="services[CPUCreditUsage]"  ' . is_checked(checkbox_binary($services["CPUCreditUsage"]), "1") . '>
                    </td>
                    <td>
                        <label for="CPUCreditUsage" class="select-cf-option">' . _('CPU Credit Usage') . '</label><br>
                        ' . _('[T2 instances] The number of CPU credits consumed by the instance') . '.
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="2" name="serviceargs[CPUCreditUsage_warning]" value="' . $serviceargs["CPUCreditUsage_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="2" name="serviceargs[CPUCreditUsage_critical]" value="' . $serviceargs["CPUCreditUsage_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" class="checkbox" id="CPUCreditBalance" name="services[CPUCreditBalance]"  ' . is_checked(checkbox_binary($services["CPUCreditBalance"]), "1") . '>
                    </td>
                    <td>
                        <label for="CPUCreditBalance" class="select-cf-option">' . _('CPU Credit Balance') . '</label><br>
                        ' . _('[T2 instances] The number of CPU credits available for the instance to burst beyond its base CPU utilization') . '.
                        <br>' . _('<b>Alerts if less than specified values because we are monitoring a balance.</b>') . '
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="2" name="serviceargs[CPUCreditBalance_warning]" value="' . $serviceargs["CPUCreditBalance_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="2" name="serviceargs[CPUCreditBalance_critical]" value="' . $serviceargs["CPUCreditBalance_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" class="checkbox" id="CPUUtilization" name="services[CPUUtilization]"  ' . is_checked(checkbox_binary($services["CPUUtilization"]), "1") . '>
                    </td>
                    <td>
                        <label for="mem" class="select-cf-option">' . _('CPU Utilization') . '</label><br>
                        ' . _('The percentage of allocated EC2 compute units that are currently in use on the instance') . '.
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="2" name="serviceargs[CPUUtilization_warning]" value="' . $serviceargs["CPUUtilization_warning"] . '" class="textfield form-control condensed"> % &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="2" name="serviceargs[CPUUtilization_critical]" value="' . $serviceargs["CPUUtilization_critical"] . '" class="textfield form-control condensed"> %
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" class="checkbox" id="DiskReadOps" name="services[DiskReadOps]"  ' . is_checked(checkbox_binary($services["DiskReadOps"]), "1") . '>
                    </td>
                    <td>
                        <label for="DiskReadOps" class="select-cf-option">' . _('Disk Read Operations') . '</label><br>
                        ' . _('The number of completed read operations from all instance store volumes available to the instace in a specified period of time') . '.
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs[DiskReadOps_warning]" value="' . $serviceargs["DiskReadOps_warning"] . '" class="textfield form-control condensed" > &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs[DiskReadOps_critical]" value="' . $serviceargs["DiskReadOps_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" class="checkbox" id="DiskWriteOps" name="services[DiskWriteOps]"  ' . is_checked(checkbox_binary($services["DiskWriteOps"]), "1") . '>
                    </td>
                    <td>
                        <label for="DiskWriteOps" class="select-cf-option">' . _('Disk Write Operations') . '</label><br>
                        ' . _('The number of completed write operations to all instance store volumes available to the instance in a specified period of time') . '.
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs[DiskWriteOps_warning]" value="' . $serviceargs["DiskWriteOps_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs[DiskWriteOps_critical]" value="' . $serviceargs["DiskWriteOps_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" class="checkbox" id="DiskReadBytes" name="services[DiskReadBytes]"  ' . is_checked(checkbox_binary($services["DiskReadBytes"]), "1") . '>
                    </td>
                    <td>
                        <label for="DiskReadBytes" class="select-cf-option">' . _('Disk Read Bytes') . '</label><br>
                        ' . _('The number of bytes read from all instance store volumes available to the instance') . '.
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs[DiskReadBytes_warning]" value="' . $serviceargs["DiskReadBytes_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs[DiskReadBytes_critical]" value="' . $serviceargs["DiskReadBytes_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" id="DiskWriteBytes" class="checkbox" name="services[DiskWriteBytes]"  ' . is_checked(checkbox_binary($services["DiskWriteBytes"]), "1") . '>
                    </td>
                    <td>
                        <label for="DiskWriteBytes" class="select-cf-option">' . _('Disk Write Bytes') . '</label><br>
                        ' . _('The number of bytes written to all instance store volumes available to the instance') . '.
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs[DiskWriteBytes_warning]" value="' . $serviceargs["DiskWriteBytes_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs[DiskWriteBytes_critical]" value="' . $serviceargs["DiskWriteBytes_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" id="NetworkIn" class="checkbox" name="services[NetworkIn]"  ' . is_checked(checkbox_binary($services["NetworkIn"]), "1") . '>
                    </td>
                    <td>
                        <label for="NetworkIn" class="select-cf-option">' . _('Network Bytes In') . '</label><br>
                        ' . _('The number of bytes received on all network interfaces by the instance') . '.
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs[NetworkIn_warning]" value="' . $serviceargs["NetworkIn_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs[NetworkIn_critical]" value="' . $serviceargs["NetworkIn_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" id="NetworkOut" class="checkbox" name="services[NetworkOut]"  ' . is_checked(checkbox_binary($services["NetworkOut"]), "1") . '>
                    </td>
                    <td>
                        <label for="NetworkOut" class="select-cf-option">' . _('Network Bytes Out') . '</label><br>
                        ' . _('The number of bytes sent out on all network interfaces by the instance') . '.
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs[NetworkOut_warning]" value="' . $serviceargs["NetworkOut_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs[NetworkOut_critical]" value="' . $serviceargs["NetworkOut_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" id="NetworkPacketsIn" class="checkbox" name="services[NetworkPacketsIn]"  ' . is_checked(checkbox_binary($services["NetworkPacketsIn"]), "1") . '>
                    </td>
                    <td>
                        <label for="NetworkPacketsIn" class="select-cf-option">' . _('Network Packets In') . '</label><br>
                        ' . _('The number of packets received on all network interfaces by the instance') . '.
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs[NetworkPacketsIn_warning]" value="' . $serviceargs["NetworkPacketsIn_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs[NetworkPacketsIn_critical]" value="' . $serviceargs["NetworkPacketsIn_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" id="NetworkPacketsOut" class="checkbox" name="services[NetworkPacketsOut]"  ' . is_checked(checkbox_binary($services["NetworkPacketsOut"]), "1") . '>
                    </td>
                    <td>
                        <label for="NetworkPacketsOut" class="select-cf-option">' . _('Network Packets Out') . '</label><br>
                        ' . _('The number of packets sent out on all network interfaces by the instance') . '.
                        <div class="pad-t5">
                            <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs[NetworkPacketsOut_warning]" value="' . $serviceargs["NetworkPacketsOut_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs[NetworkPacketsOut_critical]" value="' . $serviceargs["NetworkPacketsOut_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" id="StatusCheckFailed" class="checkbox" name="services[StatusCheckFailed]"  ' . is_checked(checkbox_binary($services["StatusCheckFailed"]), "1") . '>
                    </td>
                    <td>
                        <label for="StatusCheckFailed" class="select-cf-option">' . _('Status Check Failed') . '</label><br>
                        ' . _('Reports whether the instance has passed both the instance status check and the system status check in the last minute') . '.
                        <div class="pad-t5">
                            <input type="hidden" name="serviceargs[StatusCheckFailed_warning]" value="' . $serviceargs["StatusCheckFailed_warning"] . '" class="textfield form-control condensed"> &nbsp; <input type="hidden" name="serviceargs[StatusCheckFailed_critical]" value="' . $serviceargs["StatusCheckFailed_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" id="StatusCheckFailed_Instance" class="checkbox" name="services[StatusCheckFailed_Instance]"  ' . is_checked(checkbox_binary($services["StatusCheckFailed_Instance"]), "1") . '>
                    </td>
                    <td>
                        <label for="StatusCheckFailed_Instance" class="select-cf-option">' . _('Status Check Failed (Instance)') . '</label><br>
                        ' . _('Reports whether the instance has passed the instance status check in the last minute') . '.
                        <div class="pad-t5">
                            <input type="hidden" name="serviceargs[StatusCheckFailed_Instance_warning]" value="' . $serviceargs["StatusCheckFailed_Instance_warning"] . '" class="textfield form-control condensed"> &nbsp; <input type="hidden" name="serviceargs[StatusCheckFailed_Instance_critical]" value="' . $serviceargs["StatusCheckFailed_Instance_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

                <tr>
                    <td class="vt">
                        <input type="checkbox" id="StatusCheckFailed_System" class="checkbox" name="services[StatusCheckFailed_System]"  ' . is_checked(checkbox_binary($services["StatusCheckFailed_System"]), "1") . '>
                    </td>
                    <td>
                        <label for="StatusCheckFailed_System" class="select-cf-option">' . _('Status Check Failed (System)') . '</label><br>
                        ' . _('Reports whether the instance has passed the system status check in the last minute') . '.
                        <div class="pad-t5">
                            <input type="hidden" name="serviceargs[StatusCheckFailed_System_warning]" value="' . $serviceargs["StatusCheckFailed_System_warning"] . '" class="textfield form-control condensed"> &nbsp; <input type="hidden" name="serviceargs[StatusCheckFailed_System_critical]" value="' . $serviceargs["StatusCheckFailed_System_critical"] . '" class="textfield form-control condensed">
                        </div>
                    </td>
                </tr>

            </table>';

            $output .= '
            </table>

            <div style="height: 20px;"></div>';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE2DATA:

            // Get variables that were passed to us
            $address = grab_array_var($inargs, "address");
            $hostname = grab_array_var($inargs, "hostname");
            $instanceid = grab_array_var($inargs, "instanceid", "");
            $accesskeyid = grab_array_var($inargs, "accesskeyid", "");
            $accesskey = grab_array_var($inargs, "accesskey", "");
            $staticcreds = grab_array_var($inargs, "staticcreds", "");
            $staticconfig = grab_array_var($inargs, "staticconfig", "");
            $credsfilepath = grab_array_var($inargs, "credsfilepath", "");
            $configfilepath = grab_array_var($inargs, "configfilepath", "");
            $region = grab_array_var($inargs, "region", "");

            // Check for errors
            $errors = 0;
            $errmsg = array();
            if (is_valid_host_name($hostname) == false) {
                $errmsg[$errors++] = "Invalid host name.";
            }

            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }
            break;


        case CONFIGWIZARD_MODE_GETSTAGE3HTML:

            // Get variables that were passed to us
            $address = grab_array_var($inargs, "address");
            $hostname = grab_array_var($inargs, "hostname");
            $instanceid = grab_array_var($inargs, "instanceid", "");
            $accesskeyid = grab_array_var($inargs, "accesskeyid", "");
            $accesskey = grab_array_var($inargs, "accesskey", "");
            $staticcreds = grab_array_var($inargs, "staticcreds", "");
            $staticconfig = grab_array_var($inargs, "staticconfig", "");
            $credsfilepath = grab_array_var($inargs, "credsfilepath", "");
            $configfilepath = grab_array_var($inargs, "configfilepath", "");
            $region = grab_array_var($inargs, "region", "");


            $services = "";
            $services_serial = grab_array_var($inargs, "services_serial");
            if ($services_serial != "") {
                $services = unserialize(base64_decode($services_serial));
            } else {
                $services = grab_array_var($inargs, "services");
            }

            $serviceargs = "";
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial");
            if ($serviceargs_serial != "") {
                $serviceargs = unserialize(base64_decode($serviceargs_serial));
            } else {
                $serviceargs = grab_array_var($inargs, "serviceargs");
            }

            $output = '
            <input type="hidden" name="address" value="' . encode_form_val($address) . '">
            <input type="hidden" name="hostname" value="' . encode_form_val($hostname) . '">
            <input type="hidden" name="instanceid" value="' . encode_form_val($instanceid) . '">
            <input type="hidden" name="accesskeyid" value="' . encode_form_val($accesskeyid) . '">
            <input type="hidden" name="accesskey" value="' . encode_form_val($accesskey) . '">
            <input type="hidden" name="region" value="' . encode_form_val($region) . '">
            <input type="hidden" name="staticcreds" value="' . encode_form_val($staticcreds) . '">
            <input type="hidden" name="staticconfig" value="' . encode_form_val($staticconfig) . '">
            <input type="hidden" name="credsfilepath" value="' . encode_form_val($credsfilepath) . '">
            <input type="hidden" name="configfilepath" value="' . encode_form_val($configfilepath) . '">
            <input type="hidden" name="services_serial" value="' . base64_encode(serialize($services)) . '">
            <input type="hidden" name="serviceargs_serial" value="' . base64_encode(serialize($serviceargs)) . '">';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE3DATA:

            $check_interval = grab_array_var($inargs, "check_interval", "");

            // Check for errors
            $errors = 0;
            $errmsg = array();
            if ($check_interval < 5) {
                $errmsg[$errors++] = "Check interval cannot be less than 5 minutes.";
            }
            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }
            break;

            break;

        case CONFIGWIZARD_MODE_GETFINALSTAGEHTML:

            echo("GETFINALSTAGEHTML");

            $output = '
            ';
            break;

        case CONFIGWIZARD_MODE_GETOBJECTS:

            // Get variables that were passed to us
            $address = grab_array_var($inargs, "address");
            $hostname = grab_array_var($inargs, "hostname");
            $instanceid = grab_array_var($inargs, "instanceid", "");
            $accesskeyid = grab_array_var($inargs, "accesskeyid", "");
            $accesskey = grab_array_var($inargs, "accesskey", "");
            $staticcreds = grab_array_var($inargs, "staticcreds", "");
            $staticconfig = grab_array_var($inargs, "staticconfig", "");
            $credsfilepath = grab_array_var($inargs, "credsfilepath", "");
            $configfilepath = grab_array_var($inargs, "configfilepath", "");
            $region = grab_array_var($inargs, "region", "");
            $check_interval = grab_array_var($inargs, "check_interval", "");

            $hostaddress = $address;

            $services_serial = grab_array_var($inargs, "services_serial", "");
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", "");

            $services = unserialize(base64_decode($services_serial));
            $serviceargs = unserialize(base64_decode($serviceargs_serial));

            // Save data for later use in re-entrance
            $meta_arr = array();
            $meta_arr["hostname"] = $hostname;
            $meta_arr["address"] = $address;
            $meta_arr["instanceid"] = $instanceid;
            $meta_arr["accesskeyid"] = $accesskeyid;
            $meta_arr["accesskey"] = $accesskey;
            $meta_arr["staticcreds"] = $staticcreds;
            $meta_arr["staticconfig"] = $staticconfig;
            $meta_arr["credsfilepath"] = $credsfilepath;
            $meta_arr["configfilepath"] = $configfilepath;
            $meta_arr["region"] = $region;
            $meta_arr["services"] = $services;
            $meta_arr["serivceargs"] = $serviceargs;
            save_configwizard_object_meta($wizard_name, $hostname, "", $meta_arr);

            $objs = array();

            if (!host_exists($hostname)) {
                $objs[] = array(
                    "type" => OBJECTTYPE_HOST,
                    "use" => "xiwizard_linuxserver_host",
                    "host_name" => $hostname,
                    "address" => $hostaddress,
                    "icon_image" => "ec2.png",
                    "statusmap_image" => "ec2.png",
                    "_xiwizard" => $wizard_name,
                );
            }

            $commonopts = "--instanceid '" . $instanceid . "' ";

            // Append credentials file path if toggled, otherwise pass in instanceid, accesskeyid and accesskey
            if ($staticcreds == "on") {
                $commonopts .= "--credfile '" . $credsfilepath . "' ";
            } else {
                $commonopts .= "--accesskeyid '" . $accesskeyid . "' --secretaccesskey '" . $accesskey . "' ";
            }
            // Append config file path if toggled, otherwise pass in region
            if ($staticconfig == "on") {
                $commonopts .= "--configfile '" . $configfilepath . "' ";
            } else {
                $commonopts .= "--region '" . $region . "' ";
            }

            $service_name_dict = array(

                    "CPUCreditBalance" => "CPU Credit Balance",
                    "CPUCreditUsage" => "CPU Credit Usage",
                    "CPUUtilization" => "CPU Utilization",
                    "DiskReadBytes" => "Disk Read Bytes",
                    "DiskReadOps" => "Disk Read Ops",
                    "DiskWriteBytes" => "Disk Write Bytes",
                    "DiskWriteOps" => "Disk Write Ops",
                    "NetworkIn" => "Network In",
                    "NetworkOut" => "Network Out",
                    "NetworkPacketsIn" => "Network Packets In",
                    "NetworkPacketsOut" => "Network Packets Out",
                    "StatusCheckFailed" => "Status Check Failed",
                    "StatusCheckFailed_Instance" => "Status Check Failed Instance",
                    "StatusCheckFailed_System" => "Status Check Failed System"

                );

            // See which services we should monitor
            foreach ($services as $svc => $svcstate) {
                switch ($svc) {

                    case "ping":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Ping",
                            "use" => "xiwizard_linuxserver_ping_service",
                            "_xiwizard" => $wizard_name,
                        );
                        break;

                    case "CPUCreditBalance":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => $hostname . " " . $service_name_dict[$svc],
                            "use" => "xiwizard_linuxserver_ping_service",
                            "check_command" => "check_ec2!" . " -P " . $check_interval . " --metricname " . $svc . " " . $commonopts . "--warning '" . $serviceargs[$svc . "_warning"] . ":' --critical '" . $serviceargs[$svc . "_critical"] . ":'",
                            "_xiwizard" => $wizard_name,
                        );
                        break;

                    default:
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => $hostname . " " . $service_name_dict[$svc],
                            "use" => "xiwizard_linuxserver_ping_service",
                            "check_command" => "check_ec2!" . " -P " . $check_interval . " --metricname " . $svc . " " . $commonopts . "--warning '" . $serviceargs[$svc . "_warning"] . "' --critical '" . $serviceargs[$svc . "_critical"] . "'",
                            "_xiwizard" => $wizard_name,
                        );
                        break;
                }
            }

            // Return the object definitions to the wizard
            $outargs[CONFIGWIZARD_NAGIOS_OBJECTS] = $objs;

            break;

        default:
            break;
    }

    return $output;
}


/**
 *
 * @return string
 */
