<?php
//
// Copyright (c) 2008-2018 Nagios Enterprises, LLC. All rights reserved.
//

require_once(dirname(__FILE__) . '/phpmailer/PHPMailerAutoload.php');


////////////////////////////////////////////////////////////////////////
// EMAIL FUNCTIONS
////////////////////////////////////////////////////////////////////////


/**
 * Sends an email (HTML or text)
 *
 * @param   array   $opts       Array including: to, subject, and message
 * @param   string  $debug      The debug output
 * @param   string  $referer    Referrer to put in php mailer log
 * @param   bool    $txt        True to send message as text
 * @return  bool                True if email was sent
 * @throws  Exception 
 * @throws  phpmailerException
 */
function send_email($opts, &$debug = null, $referer = "", $txt = false)
{
    // Timestamp for phpmailer.log message
    $timestamp = date("m-d-Y H:i:s", time());

    // Make sure we have what we need
    if (!isset($opts["to"]) || !isset($opts["subject"]) || !isset($opts["message"])) {
        return false;
    }

    // Get mail options
    $mailmethod = get_option("mail_method");
    $fromaddress = get_option("mail_from_address", "root@localhost");
    $smtphost = get_option("smtp_host");
    $smtpport = get_option("smtp_port");
    $smtpusername = get_option("smtp_username");
    $smtppassword = get_option("smtp_password");
    $smtpsecurity = get_option("smtp_security");

    // Check PHPmailer debug set_option - "on"
    $debug_logging = get_option("php_sendmail_debug");

    // Generate a new instance and set to UTF-8 encoding
    $mail = new PHPMailer();
    $mail->CharSet = 'UTF-8';

    // Use global from address instead of one specified by user
    $address_parts = parse_email_address($fromaddress);
    $mail->SetFrom($address_parts[0]["email"], $address_parts[0]["name"]);
    $mail->AddReplyTo($address_parts[0]["email"], $address_parts[0]["name"]);

    // To address(es)
    $addresses = parse_email_address($opts["to"]);

    foreach ($addresses as $address) {
        $mail->AddAddress($address["email"], $address["name"]);
    }

    // Set newlines to <br> if sending as HTML
    if (!$txt) {
        $opts["message"] = nl2br($opts["message"]);
    }

    // Convert to UTF-8 if not already done so
    $mail->Subject = mb_convert_encoding($opts["subject"], "UTF-8", "auto");
    $opts["message"] = mb_convert_encoding($opts["message"], "UTF-8", "auto");

    // Set body of message
    if ($txt) {
        $mail->Subject = $opts["subject"];
        $mail->Body = $opts["message"];
        $mail->IsHTML(false);
    } else {
        $mail->MsgHTML($opts["message"]);
        $mail->IsHTML(true);
    }
    
    // For most clients expecting the Priority header:
    // 1 = High, 2 = Medium, 3 = Low
    if (isset($opts["high_priority"]) && $opts["high_priority"] == 1) {
        $mail->Priority = 1;
        // MS Outlook custom header (can be set to "Urgent" or "Highest" rather than "High")
        $mail->AddCustomHeader("X-MSMail-Priority: High");
        $mail->AddCustomHeader("Importance: High"); 
    }

    // See if there is an attachment and add to the mail
    if (isset($opts["attachment"])) {
        if (is_array($opts["attachment"])) {
            foreach ($opts["attachment"] as $aopt) {
                $mail->AddAttachment($aopt[0], $aopt[1]);
            }
        } else {
            $mail->AddAttachment($opts["attachment"]);
        }
    }

    // Use SMTP
    if ($mailmethod == "smtp") {

        $debuginfo = "method=smtp";

        $mail->IsSMTP();
        $mail->Host = $smtphost;
        $mail->Port = intval($smtpport);

        $debuginfo .= ";host=$smtphost";
        $debuginfo .= ";port=$smtpport";

        // Use SMTP Auth
        if (!empty($smtpusername)) {
            $debuginfo .= ";smtpauth=true";
            $mail->SMTPAuth = true;
            $mail->Username = $smtpusername;
            $mail->Password = $smtppassword;
        }

        // Optionally use TLS or SSL
        if ($smtpsecurity == "tls") {
            $mail->SMTPSecure = "tls";
            $debuginfo .= ";security=tls";
        } else if ($smtpsecurity == "ssl") {
            $mail->SMTPSecure = "ssl";
            $debuginfo .= ";security=ssl";
        } else {
            $debuginfo .= ";security=none";
        }
    } else {
        $debuginfo = "method=sendmail";
    }

    // Add referer to the 
    if (empty($referer)) {
        $refer_string = ", Referer: Unknown";
    } else {
        $refer_string = ", Referer: " . $referer;
    }

    // Send the email and log it in phpmailer log if we have debug turned on
    if (!$mail->Send()) {
        if (!isset($opts["debug"]))
            $debug = "[" . $timestamp . "] " . $mail->ErrorInfo . " (" . $debuginfo . ")" . $refer_string;

        if (function_exists('get_tmp_dir')) {
            if ($debug_logging == "on") {
                file_put_contents(get_tmp_dir() . '/phpmailer.log', $debug . "\n", FILE_APPEND);
            }
        }
        return false;
    } else {
        if (!isset($opts["debug"]))
            $debug = "[" . $timestamp . "] Message sent! (" . $debuginfo . ")" . $refer_string;

        if (function_exists('get_tmp_dir')) {
            if ($debug_logging == "on") {
                file_put_contents(get_tmp_dir() . '/phpmailer.log', $debug . "\n", FILE_APPEND);
            }
        }
        return true;
    }

}


/**
 * Parses a comma separated list of email addresses and returns
 * a formatted array.
 *
 * @param   string  $a  Comma-separated list of emails
 * @return  array       Array of emails
 */
function parse_email_address($a)
{
    $results = array();
    $addresses = explode(",", $a);

    foreach ($addresses as $address) {
        $newa = array(
            "name" => "",
            "email" => "",
        );

        $parts = explode("<", $address);

        // Just the address
        if (count($parts) == 1) {
            $newa["email"] = trim($parts[0]);
        } else {
            $newa["name"] = trim($parts[0]);
            $parts = explode(">", $parts[1]);
            $newa["email"] = trim($parts[0]);
        }

        $results[] = $newa;
    }

    return $results;
}

